Namespace Consultic.ExtendedControls
	Public Class TextBoxEx
		Inherits System.Windows.Forms.TextBox

		Private Const m_Caption As String = "Validation"
		Private m_OldInformation As String

		Structure ValidationEvents
			Dim ErrorMsg As String
			Dim Validated As Boolean
		End Structure

#Region "User generated events"

		Event ValidateEx(ByRef e As Consultic.ExtendedControls.TextBoxEx.ValidationEvents)

#End Region

#Region "Base class events"

		Sub New()
			MyBase.New()

			Me.BackColorNormal = MyBase.BackColor
			Me.BackColorRequired = Color.LightYellow
			Me.BackColorError = Color.DarkRed
			Me.ForeColorError = Color.White
			Me.ForeColorNormal = MyBase.ForeColor
		End Sub

		Protected Overrides Sub OnValidating(ByVal e As System.ComponentModel.CancelEventArgs)
			If Not Validate() Then
				If InformationControl Is Nothing Then
					MessageBox.Show(ErrorMessage, m_Caption, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1)
				Else
					InformationControl.Text = ErrorMessage
				End If

				Me.BackColor = Me.BackColorError
				Me.ForeColor = Me.ForeColorError

				If PreventMoveAtFailure Then
					e.Cancel = True
				End If
			Else
				If IsRequired Then
					Me.BackColor = Me.BackColorRequired
				Else
					Me.BackColor = Me.BackColorNormal
				End If
				Me.ForeColor = Me.ForeColorNormal
			End If

			MyBase.OnValidating(e)
		End Sub

		Protected Overrides Sub OnLostFocus(ByVal e As System.EventArgs)
			'If Not InformationControl Is Nothing Then InformationControl.Text = ""

			MyBase.OnLostFocus(e)
		End Sub

		Protected Overrides Sub OnGotFocus(ByVal e As System.EventArgs)
			If Not InformationControl Is Nothing Then
				If Me.ErrorMessage <> "" Then
					InformationControl.Text = Me.ErrorMessage
				Else
					InformationControl.Text = Me.InformationMessage
				End If
			End If

			MyBase.OnGotFocus(e)
		End Sub

		Protected Overrides Sub OnMouseHover(ByVal e As System.EventArgs)
			If Not InformationControl Is Nothing Then
				m_OldInformation = Me.InformationControl.Text.ToString()
				If Me.ErrorMessage <> "" Then
					InformationControl.Text = Me.ErrorMessage
				Else
					InformationControl.Text = Me.InformationMessage
				End If
			End If

			MyBase.OnMouseHover(e)
		End Sub

		Protected Overrides Sub OnMouseLeave(ByVal e As System.EventArgs)
			If Not InformationControl Is Nothing Then InformationControl.Text = m_OldInformation
			m_OldInformation = ""

			MyBase.OnMouseLeave(e)
		End Sub

		Protected Overrides Sub OnKeyUp(ByVal e As System.Windows.Forms.KeyEventArgs)
			If Me.CheckOnKeyPress And Me.CausesValidation Then
				If Not Validate() Then
					If InformationControl Is Nothing Then
						MessageBox.Show(ErrorMessage, m_Caption, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1)
					Else
						InformationControl.Text = ErrorMessage
					End If

					Me.BackColor = Me.BackColorError
					Me.ForeColor = Me.ForeColorError
				Else
					If IsRequired Then
						Me.BackColor = Me.BackColorRequired
					Else
						Me.BackColor = Me.BackColorNormal
					End If
					Me.ForeColor = Me.ForeColorNormal
					InformationControl.Text = Me.InformationMessage
				End If
			End If

			MyBase.OnKeyUp(e)
		End Sub

		'BackColor property can't be set at runtime anymore
		<Browsable(False), DesignOnly(True)> _
		Public Overrides Property BackColor() As System.Drawing.Color
			Get
				Return MyBase.BackColor
			End Get
			Set(ByVal Value As System.Drawing.Color)
				MyBase.BackColor = Value
			End Set
		End Property

		'ForeColor property can't be set at runtime anymore
		<Browsable(False), DesignOnly(True)> _
		Public Overrides Property ForeColor() As System.Drawing.Color
			Get
				Return MyBase.ForeColor
			End Get
			Set(ByVal Value As System.Drawing.Color)
				MyBase.ForeColor = Value
			End Set
		End Property

#End Region

#Region "New properties"

		Private m_MinLength As Long
		Private m_PreventMoveAtFailure As Boolean
		Private m_ErrorMessage As String
		Private m_InformationControl As Control
		Private m_InformationMessage As String
		Private m_IsRequired As Boolean
		Private m_BackColorRequired As Color
		Private m_BackColorNormal As Color
		Private m_BackColorError As Color
		Private m_ForeColorError As Color
		Private m_ForeColorNormal As Color
		Private m_CheckOnKeyPress As Boolean

		'Property MinLength, to determine the minimum length of the text in the textbox
		<Description("The minimum number of characters a textbox must contain"), Category("Extended")> _
		Property MinLength() As Long
			Get
				Return m_MinLength
			End Get
			Set(ByVal Value As Long)
				If Value <= MaxLength Then
					m_MinLength = Value
				Else
					m_MinLength = MaxLength
				End If
			End Set
		End Property

		'Propety PreventMoveAtFailure, if true, then by a failure you can't move to another control
		<Description("If true, you can't change the focus to another control on a failure"), Category("Extended")> _
		Property PreventMoveAtFailure() As Boolean
			Get
				Return m_PreventMoveAtFailure
			End Get
			Set(ByVal Value As Boolean)
				m_PreventMoveAtFailure = Value
			End Set
		End Property

		'Property (ReadOnly) ErrorMessage, to get the error message when the validation fails
		<Description("The error message shown when a failure occurs"), Category("Extended"), Browsable(False), DefaultValue(GetType(String), "")> _
		ReadOnly Property ErrorMessage() As String
			Get
				Return m_ErrorMessage
			End Get
		End Property

		'The control in which the ErrorMessage can be shown
		<Description("The control in which all the information will be shown"), Category("Extended")> _
		Property InformationControl() As Control
			Get
				Return m_InformationControl
			End Get
			Set(ByVal Value As Control)
				Dim tmp As Control

				tmp = InformationControl

				m_InformationControl = Value

				Try
					m_InformationControl.Text = ""
				Catch ex As Exception
					m_ErrorMessage = "Control doesn't expose a Text property. Control will be set back to original value"
					MessageBox.Show(ErrorMessage, m_Caption, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1)
					m_InformationControl = tmp
				End Try
			End Set
		End Property

		'The InformationControl shows this string by default, if the length of ErrorMessage is zero.
		<Description("The information what has to be typed into the textbox"), Category("Extended"), DefaultValue(GetType(String), "")> _
		Property InformationMessage() As String
			Get
				Return m_InformationMessage
			End Get
			Set(ByVal Value As String)
				m_InformationMessage = Value
			End Set
		End Property

		'The IsRequired property set to true means that a textbox can't be empty
		<Description("This property required a textbox to be filled"), Category("Extended")> _
		Property IsRequired() As Boolean
			Get
				Return m_IsRequired
			End Get
			Set(ByVal Value As Boolean)
				m_IsRequired = Value

				If m_IsRequired Then
					Me.BackColor = Me.BackColorRequired
				Else
					Me.BackColor = Me.BackColorNormal
				End If
			End Set
		End Property

		'The BackColorRequired property sets the color of the control when it is required
		<Description("This is the backcolor of the control when it is required"), Category("Extended")> _
		Property BackColorRequired() As Color
			Get
				Return m_BackColorRequired
			End Get
			Set(ByVal Value As Color)
				m_BackColorRequired = Value

				If Me.IsRequired Then
					Me.BackColor = Value
				Else
					Me.BackColor = Me.BackColorNormal
				End If
			End Set
		End Property

		'The BackColor if the control is in normal mode (means: not required)
		<Description("This is the normal backcolor (replacement of the BackColor property) of the control"), Category("Extended")> _
		Property BackColorNormal() As Color
			Get
				Return m_BackColorNormal
			End Get
			Set(ByVal Value As Color)
				m_BackColorNormal = Value

				If Not IsRequired Then
					Me.BackColor = Value
				Else
					Me.BackColor = Me.BackColorRequired
				End If
			End Set
		End Property

		'The BackColor if an error has occurred
		<Description("This is the backcolor if the control has got an error"), Category("Extended")> _
		Property BackColorError() As Color
			Get
				Return m_BackColorError
			End Get
			Set(ByVal Value As Color)
				m_BackColorError = Value

				If IsRequired Then
					Me.BackColor = Me.BackColorRequired
				Else
					Me.BackColor = Me.BackColorNormal
				End If
			End Set
		End Property

		'The ForeColor if an error has occurred
		<Description("This is the ForeColor of the textbox in case of an error"), Category("Extended")> _
		Property ForeColorError() As Color
			Get
				Return m_ForeColorError
			End Get
			Set(ByVal Value As Color)
				m_ForeColorError = Value
			End Set
		End Property

		'The ForeColor in normal situations (replacement of the ForeColor property)
		<Description("This is the normal ForeColor (Replacement of the ForeColor property) of the control"), Category("Extended")> _
		Property ForeColorNormal() As Color
			Get
				Return m_ForeColorNormal
			End Get
			Set(ByVal Value As Color)
				m_ForeColorNormal = Value
			End Set
		End Property

		'The CheckOnKeyPress property says whether every keyboard action must raise a validation
		<Description("If true, then on every keypress, the validation of the field will occur"), Category("Extended")> _
		Property CheckOnKeyPress() As Boolean
			Get
				Return m_CheckOnKeyPress
			End Get
			Set(ByVal Value As Boolean)
				m_CheckOnKeyPress = Value
			End Set
		End Property
#End Region

#Region "New functions"

		'Check if all validations are passed, if true, return true, otherwise return false
		Public Function Validate() As Boolean

			Dim e As ValidationEvents

			'Initiate Validate as True
			Validate = True

			'Check the CausesValidation property
			If Not CausesValidation Then Return True

			'Check if a value is required
			If Not CheckRequired() Then
				Validate = False
			End If

			'Check if the minimum length is reached
			If Not CheckMinLength() Then
				Validate = False
			End If

			If Validate Then m_ErrorMessage = ""

			e.Validated = Validate
			e.ErrorMsg = m_ErrorMessage

			RaiseEvent ValidateEx(e)

			m_ErrorMessage = e.ErrorMsg
			Validate = e.Validated

			Return Validate
		End Function

		'Check if the minlength is reached
		Private Function CheckMinLength() As Boolean
			If Me.Text.Length < Me.MinLength Then
				m_ErrorMessage = "The minimum length for this textbox is " & Me.MinLength.ToString() & " characters. You only typed " & Me.Text.Length.ToString() & " characters."
				Return False
			Else
				Return True
			End If
		End Function

		'Check if the textbox is required
		Private Function CheckRequired() As Boolean
			If IsRequired Then
				If Me.Text.Length > 0 Then
					Return True
				Else
					m_ErrorMessage = "This textbox is a required textbox. It must be filled."
					Return False
				End If
			Else
				Return True
			End If
		End Function

#End Region

	End Class
End Namespace